@@ -39,40 +39,38 @@ The script could be run from the command line using:
     python myscript.py ../src myapp -fc=ifort -cc=icc
 
 """
-import os
-import sys
-import traceback
-import shutil
 import datetime
 import inspect
+import os
+import shutil
+import sys
+import traceback
 
 from .config import __version__
-from .utils._Popen_wrapper import (
-    _process_Popen_initialize,
-    _process_Popen_command,
-    _process_Popen_stdout,
-    _process_Popen_communicate,
+from .utils._compiler_language_files import (
+    _get_c_files,
+    _get_fortran_files,
+    _get_ordered_srcfiles,
+    _get_srcfiles,
+    _preprocess_file,
 )
 from .utils._compiler_switches import (
-    _get_osname,
-    _get_optlevel,
     _get_c_flags,
     _get_fortran_flags,
     _get_linker_flags,
+    _get_optlevel,
     _get_os_macro,
+    _get_osname,
 )
-from .utils._compiler_language_files import (
-    _get_srcfiles,
-    _get_ordered_srcfiles,
-    _get_c_files,
-    _get_fortran_files,
-    _preprocess_file,
+from .utils._file_utils import _get_extra_exclude_files
+from .utils._meson_build import _meson_build
+from .utils._Popen_wrapper import (
+    _process_Popen_command,
+    _process_Popen_communicate,
+    _process_Popen_initialize,
+    _process_Popen_stdout,
 )
 
-# define temporary directories
-objdir_temp = os.path.join(".", "obj_temp")
-moddir_temp = os.path.join(".", "mod_temp")
-
 
 def main(
     srcdir=None,
@@ -90,6 +88,7 @@ def main(
     syslibs=None,
     arch="intel64",
     makefile=False,
+    makefiledir=".",
     srcdir2=None,
     extrafiles=None,
     excludefiles=None,
@@ -98,6 +97,8 @@ def main(
     verbose=False,
     inplace=False,
     networkx=False,
+    meson=False,
+    mesondir=".",
 ):
     """Main pymake function.
 
@@ -138,6 +139,8 @@ def main(
         Architecture to use for Intel Compilers on Windows (default is intel64)
     makefile : bool
         boolean indicating if a GNU make makefile should be created
+    makefiledir : str
+        GNU make makefile path
     srcdir2 : str
         additional directory with common source files.
     extrafiles : str
@@ -154,7 +157,7 @@ def main(
         boolean indicating if output will be printed to the terminal
     inplace : bool
         boolean indicating that the source files in srcdir, srcdir2, and
-        defined in extrafiles will be used directly. If inplace is True,
+        defined in extrafiles will be used directly. If inplace is False,
         source files will be copied to a directory named srcdir_temp.
         (default is False)
     networkx : bool
@@ -163,6 +166,11 @@ def main(
         source files are compiled in. The NetworkX package tends to result in
         a unique DAG more often than the standard algorithm used in pymake.
         (default is False)
+    meson : bool
+        boolean indicating that the executable should be built using the
+        meson build system. (default is False)
+    mesondir : str
+        Main meson.build file path
 
     Returns
     -------
@@ -171,12 +179,21 @@ def main(
 
     """
 
+    if meson:
+        if not inplace:
+            inplace = True
+            print(
+                f"Using meson to build {os.path.basename(target)}, "
+                + "ressetting inplace to True"
+            )
+
     if srcdir is not None and target is not None:
 
+        objdir_temp, moddir_temp, srcdir_temp = get_temporary_directories(
+            appdir=appdir
+        )
         if inplace:
             srcdir_temp = srcdir
-        else:
-            srcdir_temp = os.path.join(".", "src_temp")
 
         # process appdir
         if appdir is not None:
@@ -216,12 +233,10 @@ def main(
 
         # write summary information
         if verbose:
-            print("\nsource files are in:\n    {}\n".format(srcdir))
-            print("executable name to be created:\n    {}\n".format(target))
+            print(f"\nsource files are in:\n    {srcdir}\n")
+            print(f"executable name to be created:\n    {target}\n")
             if srcdir2 is not None:
-                msg = "additional source files are in:\n" + "     {}\n".format(
-                    srcdir2
-                )
+                msg = "additional source files are in:\n" + f"     {srcdir2}\n"
                 print(msg)
 
         # make sure the path for the target exists
@@ -229,7 +244,7 @@ def main(
         if pth == "":
             pth = "."
         if not os.path.exists(pth):
-            print("creating target path - {}\n".format(pth))
+            print(f"creating target path - {pth}\n")
             os.makedirs(pth)
 
         # initialize
@@ -240,7 +255,10 @@ def main(
             extrafiles,
             excludefiles,
             include_subdirs,
+            objdir_temp,
+            moddir_temp,
             srcdir_temp,
+            meson,
         )
 
         # get ordered list of files to compile
@@ -248,36 +266,56 @@ def main(
 
         # set intelwin flag to True in compiling on windows with Intel compilers
         intelwin = False
-        if _get_osname() == "win32":
-            if fc is not None:
-                if fc in ["ifort", "mpiifort"]:
-                    intelwin = True
-            if cc is not None:
-                if cc in ["cl", "icl"]:
-                    intelwin = True
+        if not meson:
+            if _get_osname() == "win32":
+                if fc is not None:
+                    if fc in ["ifort", "mpiifort"]:
+                        intelwin = True
+                if cc is not None:
+                    if cc in ["cl", "icl"]:
+                        intelwin = True
 
         # update openspec files based on intelwin
         if not intelwin:
             _create_openspec(srcfiles, verbose)
 
         # compile the executable
-        returncode = _pymake_compile(
-            srcfiles,
-            target,
-            fc,
-            cc,
-            expedite,
-            dryrun,
-            double,
-            debug,
-            fflags,
-            cflags,
-            syslibs,
-            arch,
-            intelwin,
-            sharedobject,
-            verbose,
-        )
+        if meson:
+            returncode = _meson_build(
+                target,
+                srcdir,
+                srcdir2,
+                extrafiles,
+                srcfiles,
+                debug,
+                double,
+                fc,
+                cc,
+                fflags,
+                cflags,
+                syslibs,
+                sharedobject,
+                mesondir,
+                verbose,
+            )
+        else:
+            returncode = _pymake_compile(
+                srcfiles,
+                target,
+                fc,
+                cc,
+                expedite,
+                dryrun,
+                double,
+                debug,
+                fflags,
+                cflags,
+                syslibs,
+                arch,
+                intelwin,
+                sharedobject,
+                verbose,
+            )
 
         # create makefile
         if makefile:
@@ -294,16 +332,28 @@ def main(
                 fflags,
                 cflags,
                 syslibs,
+                sharedobject,
+                makefiledir,
                 verbose,
             )
 
         # clean up temporary files
         if makeclean and returncode == 0:
-            _clean_temp_files(target, intelwin, inplace, srcdir_temp, verbose)
+            _clean_temp_files(
+                target,
+                intelwin,
+                inplace,
+                objdir_temp,
+                moddir_temp,
+                srcdir_temp,
+                meson,
+                mesondir,
+                verbose,
+            )
     else:
         msg = (
-            "Nothing to do, the srcdir ({}) ".format(srcdir)
-            + "and/or target ({}) ".format(target)
+            f"Nothing to do, the srcdir ({srcdir}) "
+            + f"and/or target ({target}) "
             + "are not specified."
         )
         raise ValueError(msg)
@@ -318,7 +368,10 @@ def _pymake_initialize(
     extrafiles,
     excludefiles,
     include_subdirs,
+    objdir_temp,
+    moddir_temp,
     srcdir_temp,
+    meson,
 ):
     """Remove temp source directory and target, and then copy source into
     source temp directory.
@@ -340,6 +393,10 @@ def _pymake_initialize(
     include_subdirs : bool
         boolean indicating source files in srcdir subdirectories should be
         included in the build
+    objdir_temp : str
+        path for temporary directory that will contain the object files.
+    moddir_temp : str
+        path for temporary directory that will contain the module files.
     srcdir_temp : str
         path for directory that will contain the source files. If
         srcdir_temp is the same as srcdir then the original source files
@@ -399,7 +456,7 @@ def _pymake_initialize(
             else:
                 shutil.copytree(src, dst)
         else:
-            dst = os.path.normpath(os.path.relpath(commonsrc, srcdir_temp))
+            dst = os.path.relpath(os.path.abspath(os.path.abspath(commonsrc)))
 
         srcfiles += _get_srcfiles(dst, include_subdirs)
 
@@ -417,9 +474,9 @@ def _pymake_initialize(
                 if os.path.isfile(fpth):
                     fpth = fpth2
                 else:
-                    msg = "Current working directory: {}\n".format(os.getcwd())
-                    msg += "Error in extrafiles: {}\n".format(extrafiles)
-                    msg += "Could not find file: {}".format(fpth)
+                    msg = f"Current working directory: {os.getcwd()}\n"
+                    msg += f"Error in extrafiles: {extrafiles}\n"
+                    msg += f"Could not find file: {fpth}"
                     raise FileNotFoundError(msg)
         if inplace:
             dst = os.path.normpath(os.path.relpath(fpth, os.getcwd()))
@@ -428,7 +485,7 @@ def _pymake_initialize(
             if os.path.isfile(dst):
                 raise ValueError(
                     "Error with extrafile.  Name conflicts with "
-                    "an existing source file: {}".format(dst)
+                    f"an existing source file: {dst}"
                 )
         if not inplace:
             shutil.copy(fpth, dst)
@@ -445,54 +502,57 @@ def _pymake_initialize(
         for fpth in remove_list:
             srcfiles.remove(fpth)
 
-    # if they don't exist, create directories for objects and mods
-    if not os.path.exists(objdir_temp):
-        os.makedirs(objdir_temp)
-    if not os.path.exists(moddir_temp):
-        os.makedirs(moddir_temp)
+    # if they don't exist and not compiling with meson,
+    # create directories for objects and module (*.mod) files.
+    if not meson:
+        if not os.path.exists(objdir_temp):
+            os.makedirs(objdir_temp)
+        if not os.path.exists(moddir_temp):
+            os.makedirs(moddir_temp)
 
     return srcfiles
 
 
-def _get_extra_exclude_files(extrafiles):
-    """Get extrafiles to include in compilation from a file or a list.
+def get_temporary_directories(appdir=None):
+    """Get paths to temporary object, module, and source files
 
     Parameters
     ----------
-    extrafiles : str
-        path for extrafiles file that contains paths to additional source
-        files to include
+    appdir : str
+        path for executable
 
     Returns
     -------
-    files : list
-        list of files in the extra files input file
+    obj_temp : str
+        path to temporary object files
+    mod_temp : str
+        path to temporary module files
+    src_temp : str
+        path to temporary source files
 
     """
-    if extrafiles is None:
-        files = None
+    if appdir is None:
+        base_pth = "."
     else:
-        if isinstance(extrafiles, list):
-            files = extrafiles
-        elif os.path.isfile(extrafiles):
-            efpth = os.path.dirname(extrafiles)
-            with open(extrafiles, "r") as f:
-                files = []
-                for line in f:
-                    fname = line.strip().replace("\\", "/")
-                    if len(fname) > 0:
-                        fname = os.path.abspath(os.path.join(efpth, fname))
-                        files.append(fname)
-        else:
-            raise Exception(
-                "extrafiles must be either a list of files "
-                "or the name of a text file that contains a list "
-                "of files."
-            )
-    return files
+        base_pth = appdir
+    return (
+        os.path.join(base_pth, "obj_temp"),
+        os.path.join(base_pth, "mod_temp"),
+        os.path.join(base_pth, "src_temp"),
+    )
 
 
-def _clean_temp_files(target, intelwin, inplace, srcdir_temp, verbose=False):
+def _clean_temp_files(
+    target,
+    intelwin,
+    inplace,
+    objdir_temp,
+    moddir_temp,
+    srcdir_temp,
+    meson,
+    mesondir,
+    verbose=False,
+):
     """Cleanup intermediate files. Remove mod and object files, and remove the
     temporary source directory.
 
@@ -508,10 +568,19 @@ def _clean_temp_files(target, intelwin, inplace, srcdir_temp, verbose=False):
         defined in extrafiles will be used directly. If inplace is True,
         source files will be copied to a directory named srcdir_temp.
         (default is False)
+    objdir_temp : str
+        path for temporary directory that will contain the object files.
+    moddir_temp : str
+        path for temporary directory that will contain the module files.
     srcdir_temp : str
         path for directory that will contain the source files. If
         srcdir_temp is the same as srcdir then the original source files
         will be used.
+    meson : bool
+        boolean indicating that the executable should be built using the
+        meson build system. (default is False)
+    mesondir : str
+        Main meson.build file path
     verbose : bool
         boolean indicating if output will be printed to the terminal
 
@@ -535,7 +604,7 @@ def _clean_temp_files(target, intelwin, inplace, srcdir_temp, verbose=False):
         for ext in delext:
             if f.endswith(ext):
                 if verbose:
-                    print("    removing...{}".format(f))
+                    print(f"    removing...{f}")
                 os.remove(f)
 
     # shared object intermediate files
@@ -548,7 +617,7 @@ def _clean_temp_files(target, intelwin, inplace, srcdir_temp, verbose=False):
         for ext in delext:
             if fpth.endswith(ext):
                 if verbose:
-                    print("    removing...'{}'".format(fpth))
+                    print(f"    removing...'{fpth}'")
                 os.remove(fpth)
 
     # remove temporary directories
@@ -561,20 +630,32 @@ def _clean_temp_files(target, intelwin, inplace, srcdir_temp, verbose=False):
     if not inplace:
         if os.path.isdir(srcdir_temp):
             if verbose:
-                print("removing...'{}'".format(srcdir_temp))
+                print(f"removing...'{srcdir_temp}'")
             shutil.rmtree(srcdir_temp)
     if os.path.isdir(objdir_temp):
         if verbose:
-            print("removing...'{}'".format(objdir_temp))
+            print(f"removing...'{objdir_temp}'")
         shutil.rmtree(objdir_temp)
     if os.path.isdir(moddir_temp):
         if verbose:
-            print("removing...'{}'".format(moddir_temp))
+            print(f"removing...'{moddir_temp}'")
         shutil.rmtree(moddir_temp)
+    if meson:
+        meson_builddir = os.path.join(mesondir, "_build")
+        if os.path.isdir(meson_builddir):
+            if verbose:
+                print(f"removing...'{meson_builddir}'")
+            shutil.rmtree(meson_builddir)
+        main_meson_file = os.path.join(mesondir, "meson.build")
+        if os.path.isfile(main_meson_file):
+            if verbose:
+                print(f"removing...'{main_meson_file}'")
+                os.remove(main_meson_file)
 
     # remove the windows batchfile
-    if intelwin:
-        os.remove("compile.bat")
+    batch_file = "compile.bat"
+    if intelwin and os.path.isfile(batch_file):
+        os.remove(batch_file)
     return
 
 
@@ -607,7 +688,7 @@ def _create_openspec(srcfiles, verbose):
             fpth = os.path.join(dpth, file)
             if os.path.isfile(fpth):
                 if verbose:
-                    print('replacing..."{}"'.format(fpth))
+                    print(f'replacing..."{fpth}"')
                 f = open(fpth, "w")
                 line = (
                     "c -- created by pymake_base.py\n"
@@ -703,11 +784,6 @@ def _pymake_compile(
         boolean indicating a shared object will be built
     verbose : bool
         boolean indicating if output will be printed to the terminal
-    inplace : bool
-        boolean indicating that the source files in srcdir, srcdir2, and
-        defined in extrafiles will be used directly. If inplace is True,
-        source files will be copied to a directory named srcdir_temp.
-        (default is False)
 
     Returns
     -------
@@ -717,10 +793,7 @@ def _pymake_compile(
     """
     # write pymake setting
     if verbose:
-        msg = (
-            "\nPymake settings in {}\n".format(_pymake_compile.__name__)
-            + 40 * "-"
-        )
+        msg = f"\nPymake settings in {_pymake_compile.__name__}\n" + 40 * "-"
         print(msg)
         frame = inspect.currentframe()
         fnargs, _, _, values = inspect.getargvalues(frame)
@@ -730,7 +803,7 @@ def _pymake_compile(
                 value = "None"
             elif isinstance(value, list):
                 value = ", ".join(value)
-            print(" {}={}".format(arg, value))
+            print(f" {arg}={value}")
 
     # initialize returncode
     returncode = 0
@@ -738,6 +811,11 @@ def _pymake_compile(
     # initialize ilink
     ilink = 0
 
+    # get temporary object and module directories
+    objdir_temp, moddir_temp, _ = get_temporary_directories(
+        os.path.dirname(target)
+    )
+
     # set optimization levels
     optlevel = _get_optlevel(
         target, fc, cc, debug, fflags, cflags, verbose=verbose
@@ -778,7 +856,7 @@ def _pymake_compile(
     # non-zero error code
     if os.path.isfile(target):
         if verbose:
-            msg = "removing existing target with same name: {}".format(target)
+            msg = f"removing existing target with same name: {target}"
             print(msg)
         os.remove(target)
 
@@ -810,7 +888,7 @@ def _pymake_compile(
                 os.remove(batchfile)
             except:
                 if verbose:
-                    print("could not remove '{}'".format(batchfile))
+                    print(f"could not remove '{batchfile}'")
 
         # Create target using a batch file on Windows
         try:
@@ -823,6 +901,8 @@ def _pymake_compile(
                 tfflags,
                 tcflags,
                 tlflags,
+                objdir_temp,
+                moddir_temp,
                 srcfiles,
                 target,
                 arch,
@@ -834,7 +914,7 @@ def _pymake_compile(
                 batchfile,
             ]
         except:
-            errmsg = "Could not make x64 target: {}\n".format(target)
+            errmsg = f"Could not make x64 target: {target}\n"
             errmsg += traceback.print_exc()
             print(errmsg)
 
@@ -891,15 +971,15 @@ def _pymake_compile(
             # add search path for any c and c++ header files
             if iscfile:
                 for sd in searchdir:
-                    cmdlist.append("-I{}".format(sd))
+                    cmdlist.append(f"-I{sd}")
             # put object files and module files in objdir_temp and moddir_temp
             else:
-                cmdlist.append("-I{}".format(objdir_temp))
+                cmdlist.append(f"-I{objdir_temp}")
                 if fc in ["ifort", "mpiifort"]:
                     cmdlist.append("-module")
                     cmdlist.append(moddir_temp + "/")
                 else:
-                    cmdlist.append("-J{}".format(moddir_temp))
+                    cmdlist.append(f"-J{moddir_temp}")
 
             cmdlist.append("-c")
             cmdlist.append(srcfile)
@@ -946,17 +1026,19 @@ def _pymake_compile(
             if idx == 0:
                 if intelwin:
                     msg = (
-                        "\nCompiling '{}' ".format(os.path.basename(target))
+                        f"\nCompiling '{os.path.basename(target)}' "
                         + "for Windows using Intel compilers..."
                     )
                 else:
-                    msg = "\nCompiling object files for " + "'{}'...".format(
-                        os.path.basename(target)
+                    msg = (
+                        "\nCompiling object files for "
+                        + f"'{os.path.basename(target)}'"
                     )
                 print(msg)
             if idx > 0 and idx == ilink:
-                msg = "\nLinking object files " + "to make '{}'...".format(
-                    os.path.basename(target)
+                msg = (
+                    "\nLinking object files "
+                    + f"to make '{os.path.basename(target)}'..."
                 )
                 print(msg)
 
@@ -976,7 +1058,7 @@ def _pymake_compile(
             # evaluate return code
             returncode = proc.returncode
             if returncode != 0:
-                msg = "compilation failed on '{}'".format(" ".join(cmdlist))
+                msg = f"compilation failed on '{' '.join(cmdlist)}'"
                 print(msg)
                 break
 
@@ -996,6 +1078,8 @@ def _create_win_batch(
     fflags,
     cflags,
     lflags,
+    objdir_temp,
+    moddir_temp,
     srcfiles,
     target,
     arch,
@@ -1022,6 +1106,10 @@ def _create_win_batch(
     lflags : list
         linker compiler flags, which are a combination of user provided list
         of compiler flags for the compiler to used for linking
+    objdir_temp : str
+        path for temporary directory that will contain the object files.
+    moddir_temp : str
+        path for temporary directory that will contain the module files.
     srcfiles : list
         list of source file names
     target : str
@@ -1043,19 +1131,19 @@ def _create_win_batch(
             if on_env_var == oneapi_list[0]:
                 cpvars = (
                     "C:\\Program Files (x86)\\Intel\\oneAPI\\compiler\\"
-                    + "{}\\env\\vars.bat".format(latest_version)
+                    + f"{latest_version}\\env\\vars.bat"
                 )
             else:
                 cpvars = (
                     "C:\\Program Files (x86)\\Intel\\oneAPI\\" + "setvars.bat"
                 )
             if not os.path.isfile(cpvars):
-                raise Exception("Could not find cpvars: {}".format(cpvars))
-            intel_setvars = '"{}"'.format(cpvars)
+                raise Exception(f"Could not find cpvars: {cpvars}")
+            intel_setvars = f'"{cpvars}"'
             break
     # stand alone intel installation
     if intel_setvars is None:
-        iflist = ["IFORT_COMPILER{}".format(i) for i in range(30, 12, -1)]
+        iflist = [f"IFORT_COMPILER{i}" for i in range(30, 12, -1)]
         for ift in iflist:
             stand_alone_intel = os.environ.get(ift)
             if stand_alone_intel is not None:
@@ -1063,7 +1151,7 @@ def _create_win_batch(
                     stand_alone_intel, "bin", "compilervars.bat"
                 )
                 if not os.path.isfile(cpvars):
-                    raise Exception("Could not find cpvars: {}".format(cpvars))
+                    raise Exception(f"Could not find cpvars: {cpvars}")
                 intel_setvars = '"' + os.path.normpath(cpvars) + '" ' + arch
                 break
     # check if either OneAPI or stand alone intel is installed
@@ -1104,7 +1192,7 @@ def _create_win_batch(
 
             # add search path for any header files
             for sd in searchdir:
-                cmd += "/I{} ".format(sd)
+                cmd += f"/I{sd} "
 
             obj = os.path.join(
                 objdir_temp,
@@ -1120,10 +1208,10 @@ def _create_win_batch(
             if _preprocess_file(srcfile):
                 cmd += "/fpp" + " "
             cmd += "/c" + " "
-            cmd += "/module:{0}\\ ".format(moddir_temp)
-            cmd += "/object:{0}\\ ".format(objdir_temp)
+            cmd += f"/module:{moddir_temp}\\ "
+            cmd += f"/object:{objdir_temp}\\ "
             cmd += srcfile
-        f.write("echo {}\n".format(cmd))
+        f.write(f"echo {cmd}\n")
         f.write(cmd + "\n")
 
     # write commands to link
@@ -1140,7 +1228,7 @@ def _create_win_batch(
     for switch in lflags:
         cmd += " " + switch
     cmd += "\n"
-    f.write("echo {}\n".format(cmd))
+    f.write(f"echo {cmd}\n")
     f.write(cmd)
 
     # close the batch file
@@ -1162,6 +1250,8 @@ def _create_makefile(
     fflags,
     cflags,
     syslibs,
+    sharedobject,
+    makefiledir,
     verbose,
     makedefaults="makedefaults",
 ):
@@ -1195,6 +1285,12 @@ def _create_makefile(
         user provided list of c or cpp compiler flags
     syslibs : list
         user provided syslibs
+    sharedobject : bool
+        boolean indicating a shared object will be built
+    makefiledir : str
+        GNU make makefile path
+    verbose : bool
+        boolean indicating if output will be printed to the terminal
     makedefaults : str
         name of the makedefaults file to create with makefile (default is
         makedefaults)
@@ -1205,9 +1301,25 @@ def _create_makefile(
     """
     # write a message
     if verbose:
-        msg = "\nWriting makefile and {}".format(makedefaults)
+        msg = f"\nWriting makefile and {makedefaults}"
         print(msg)
 
+    # set executable extension
+    if sharedobject:
+        win_ext = ".dll"
+        macos_ext = ".dylib"
+        linux_ext = ".so"
+    else:
+        win_ext = ".exe"
+        macos_ext = ""
+        linux_ext = ""
+
+    # set makefile directory
+    make_dir = makefiledir
+
+    # get temporary directories
+    objdir_temp, moddir_temp, _ = get_temporary_directories(make_dir)
+
     # set object extension
     objext = ".o"
 
@@ -1231,7 +1343,7 @@ def _create_makefile(
     )
 
     # open makefile
-    f = open("makefile", "w")
+    f = open(os.path.join(make_dir, "makefile"), "w")
 
     # write header
     f.write(heading + "\n")
@@ -1262,8 +1374,9 @@ def _create_makefile(
     f.write(line)
     vpaths = []
     for idx, source_dir in enumerate(dirs):
+        rel_source_dir = os.path.relpath(source_dir, make_dir)
         vpaths.append(f"SOURCEDIR{idx + 1}")
-        line = f"{vpaths[idx]}={source_dir}\n"
+        line = f"{vpaths[idx]}={rel_source_dir}\n"
         f.write(line)
     f.write("\n")
 
@@ -1303,7 +1416,7 @@ def _create_makefile(
         line = "\t-$(CC) $(OPTLEVEL) -o $@ $(OBJECTS) $(LDFLAGS)\n"
     else:
         line = "\t-$(FC) $(OPTLEVEL) -o $@ $(OBJECTS) $(LDFLAGS)\n"
-    f.write("{}\n".format(line))
+    f.write(f"{line}\n")
 
     if fext is not None:
         for ext in fext:
@@ -1322,13 +1435,13 @@ def _create_makefile(
             line = (
                 "\t$(CC) $(OPTLEVEL) $(CFLAGS) -c $< -o $@ " + "$(INCSWITCH)\n"
             )
-            f.write("{}\n".format(line))
+            f.write(f"{line}\n")
 
     # close the makefile
     f.close()
 
     # open makedefaults
-    f = open(makedefaults, "w")
+    f = open(os.path.join(make_dir, makedefaults), "w")
 
     # replace makefile in heading with makedefaults
     heading = heading.replace("makefile", makedefaults)
@@ -1357,32 +1470,32 @@ def _create_makefile(
     # get path to executable
     dpth = os.path.dirname(target)
     if len(dpth) > 0:
-        dpth = os.path.relpath(dpth)
+        dpth = os.path.relpath(dpth, make_dir)
     else:
         dpth = "."
 
-    # write
+    # write header
     line = (
         "# Define the directories for the object and module files\n"
         + "# and the executable and its path.\n"
     )
     tpth = dpth.replace("\\", "/")
     line += f"BINDIR = {tpth}\n"
-    tpth = objdir_temp.replace("\\", "/")
+    tpth = os.path.relpath(objdir_temp.replace("\\", "/"), make_dir)
     line += f"OBJDIR = {tpth}\n"
-    tpth = moddir_temp.replace("\\", "/")
+    tpth = os.path.relpath(moddir_temp.replace("\\", "/"), make_dir)
     line += f"MODDIR = {tpth}\n"
     line += "INCSWITCH = -I $(OBJDIR)\n"
     line += "MODSWITCH = -J $(MODDIR)\n\n"
     f.write(line)
 
-    line = "# define program name\n"
-    line += f"PROGRAM = $(BINDIR)/{exe_name}\n\n"
-    f.write(line)
-
     line = "# define os dependent program name\n"
     line += "ifeq ($(detected_OS), Windows)\n"
-    line += f"\tPROGRAM = $(BINDIR)/{exe_name}.exe\n"
+    line += f"\tPROGRAM = $(BINDIR)/{exe_name}{win_ext}\n"
+    line += "else ifeq ($(detected_OS), Darwin)\n"
+    line += f"\tPROGRAM = $(BINDIR)/{exe_name}{macos_ext}\n"
+    line += "else\n"
+    line += f"\tPROGRAM = $(BINDIR)/{exe_name}{linux_ext}\n"
     line += "endif\n\n"
     f.write(line)
 
@@ -1429,7 +1542,7 @@ def _create_makefile(
         target, fc, cc, debug, fflags, cflags, verbose=verbose
     )
     line = "# set the optimization level (OPTLEVEL) if not defined\n"
-    line += "OPTLEVEL ?= {}\n\n".format(optlevel.replace("/", "-"))
+    line += f"OPTLEVEL ?= {optlevel.replace('/', '-')}\n\n"
     f.write(line)
 
     # fortran flags
@@ -1451,6 +1564,7 @@ def _create_makefile(
             debug,
             double,
             osname="win32",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
         for idx, flag in enumerate(tfflags):
@@ -1458,7 +1572,7 @@ def _create_makefile(
                 tfflags[idx] = "$(OS_macro)"
         if preprocess:
             tfflags.append("-cpp")
-        line += "\t\tFFLAGS ?= {}\n".format(" ".join(tfflags))
+        line += f"\t\tFFLAGS ?= {' '.join(tfflags)}\n"
         line += "\tendif\n"
         line += "else\n"
         line += "\tifeq ($(FC), gfortran)\n"
@@ -1469,6 +1583,7 @@ def _create_makefile(
             debug,
             double,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
         for idx, flag in enumerate(tfflags):
@@ -1476,7 +1591,7 @@ def _create_makefile(
                 tfflags[idx] = "$(OS_macro)"
         if preprocess:
             tfflags.append("-cpp")
-        line += "\t\tFFLAGS ?= {}\n".format(" ".join(tfflags))
+        line += f"\t\tFFLAGS ?= {' '.join(tfflags)}\n"
         line += "\tendif\n"
         line += "\tifeq ($(FC), $(filter $(FC), ifort mpiifort))\n"
         tfflags = _get_fortran_flags(
@@ -1486,6 +1601,7 @@ def _create_makefile(
             debug,
             double,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
         for idx, flag in enumerate(tfflags):
@@ -1493,9 +1609,25 @@ def _create_makefile(
                 tfflags[idx] = "$(OS_macro)"
         if preprocess:
             tfflags.append("-fpp")
-        line += "\t\tFFLAGS ?= {}\n".format(" ".join(tfflags))
+        line += f"\t\tFFLAGS ?= {' '.join(tfflags)}\n"
         line += "\t\tMODSWITCH = -module $(MODDIR)\n"
         line += "\tendif\n"
+        line += "\tifeq ($(FC), $(filter $(FC), ftn))\n"
+        tfflags = _get_fortran_flags(
+            target,
+            "ftn",
+            [],
+            debug,
+            double,
+            osname="linux",
+            sharedobject=sharedobject,
+            verbose=verbose,
+        )
+        for idx, flag in enumerate(tfflags):
+            if "-D__" in flag:
+                tfflags[idx] = "$(OS_macro)"
+        line += f"\t\tFFLAGS ?= {' '.join(tfflags)}\n"
+        line += "\tendif\n"
         line += "endif\n\n"
         f.write(line)
 
@@ -1511,9 +1643,10 @@ def _create_makefile(
             debug,
             srcfiles,
             osname="win32",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
-        line += "\t\tCFLAGS ?= {}\n".format(" ".join(tcflags))
+        line += f"\t\tCFLAGS ?= {' '.join(tcflags)}\n"
         line += "\tendif\n"
         line += "\tifeq ($(CC), $(filter $(CC), clang clang++))\n"
         tcflags = _get_c_flags(
@@ -1523,9 +1656,10 @@ def _create_makefile(
             debug,
             srcfiles,
             osname="win32",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
-        line += "\t\tCFLAGS ?= {}\n".format(" ".join(tcflags))
+        line += f"\t\tCFLAGS ?= {' '.join(tcflags)}\n"
         line += "\tendif\n"
         line += "else\n"
         line += "\tifeq ($(CC), $(filter $(CC), gcc g++))\n"
@@ -1536,9 +1670,10 @@ def _create_makefile(
             debug,
             srcfiles,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
-        line += "\t\tCFLAGS ?= {}\n".format(" ".join(tcflags))
+        line += f"\t\tCFLAGS ?= {' '.join(tcflags)}\n"
         line += "\tendif\n"
         line += "\tifeq ($(CC), $(filter $(CC), clang clang++))\n"
         tcflags = _get_c_flags(
@@ -1548,9 +1683,10 @@ def _create_makefile(
             debug,
             srcfiles,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
-        line += "\t\tCFLAGS ?= {}\n".format(" ".join(tcflags))
+        line += f"\t\tCFLAGS ?= {' '.join(tcflags)}\n"
         line += "\tendif\n"
         line += "\tifeq ($(CC), $(filter $(CC), icc mpiicc icpc))\n"
         tcflags = _get_c_flags(
@@ -1560,9 +1696,10 @@ def _create_makefile(
             debug,
             srcfiles,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
-        line += "\t\tCFLAGS ?= {}\n".format(" ".join(tcflags))
+        line += f"\t\tCFLAGS ?= {' '.join(tcflags)}\n"
         line += "\tendif\n"
         line += "endif\n\n"
         f.write(line)
@@ -1580,10 +1717,11 @@ def _create_makefile(
             [],
             srcfiles,
             osname="win32",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
         line += "\tifeq ($(CC), $(filter $(CC), gcc g++))\n"
-        line += "\t\tLDFLAGS ?= {}\n".format(" ".join(tsyslibs))
+        line += f"\t\tLDFLAGS ?= {' '.join(tsyslibs)}\n"
         line += "\tendif\n"
         _, tsyslibs = _get_linker_flags(
             target,
@@ -1592,10 +1730,11 @@ def _create_makefile(
             [],
             srcfiles,
             osname="win32",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
         line += "\tifeq ($(CC), $(filter $(CC), clang clang++))\n"
-        line += "\t\tLDFLAGS ?= {}\n".format(" ".join(tsyslibs))
+        line += f"\t\tLDFLAGS ?= {' '.join(tsyslibs)}\n"
         line += "\tendif\n"
     # fortran compiler used for linking
     else:
@@ -1606,10 +1745,11 @@ def _create_makefile(
             [],
             srcfiles,
             osname="win32",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
         line += "\tifeq ($(FC), $(filter $(FC), gfortran))\n"
-        line += "\t\tLDFLAGS ?= {}\n".format(" ".join(tsyslibs))
+        line += f"\t\tLDFLAGS ?= {' '.join(tsyslibs)}\n"
         line += "\tendif\n"
     # linux and osx
     line += "else\n"
@@ -1622,10 +1762,11 @@ def _create_makefile(
             [],
             srcfiles,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
         line += "\tifeq ($(CC), $(filter $(CC), gcc g++))\n"
-        line += "\t\tLDFLAGS ?= {}\n".format(" ".join(tsyslibs))
+        line += f"\t\tLDFLAGS ?= {' '.join(tsyslibs)}\n"
         line += "\tendif\n"
         _, tsyslibs = _get_linker_flags(
             target,
@@ -1634,10 +1775,11 @@ def _create_makefile(
             [],
             srcfiles,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
         line += "\tifeq ($(CC), $(filter $(CC), clang clang++))\n"
-        line += "\t\tLDFLAGS ?= {}\n".format(" ".join(tsyslibs))
+        line += f"\t\tLDFLAGS ?= {' '.join(tsyslibs)}\n"
         line += "\tendif\n"
     # fortran compiler used for linking
     else:
@@ -1650,9 +1792,10 @@ def _create_makefile(
             [],
             srcfiles,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
-        line += "\t\tLDFLAGS ?= {}\n".format(" ".join(tsyslibs))
+        line += f"\t\tLDFLAGS ?= {' '.join(tsyslibs)}\n"
         line += "\tendif\n"
         # ifort compiler
         line += "\tifeq ($(FC), $(filter $(FC), ifort mpiifort))\n"
@@ -1663,10 +1806,26 @@ def _create_makefile(
             [],
             srcfiles,
             osname="linux",
+            sharedobject=sharedobject,
             verbose=verbose,
         )
-        line += "\t\tLDFLAGS ?= {}\n".format(" ".join(tsyslibs))
+        line += f"\t\tLDFLAGS ?= {' '.join(tsyslibs)}\n"
         line += "\tendif\n"
+        # ftn compiler
+        line += "\tifeq ($(FC), $(filter $(FC), ftn))\n"
+        _, tsyslibs = _get_linker_flags(
+            target,
+            "ftn",
+            "clang",
+            [],
+            srcfiles,
+            osname="linux",
+            sharedobject=sharedobject,
+            verbose=verbose,
+        )
+        line += f"\t\tLDFLAGS ?= {' '.join(tsyslibs)}\n"
+        line += "\tendif\n"
+
     line += "endif\n\n"
     f.write(line)
 
@@ -1732,7 +1891,10 @@ def _create_makefile(
     if sys.platform == "win32":
         windows_line_ending = b"\r\n"
         unix_line_ending = b"\n"
-        for file in ("makefile", makedefaults):
+        for file in (
+            os.path.join(make_dir, "makefile"),
+            os.path.join(make_dir, makedefaults),
+        ):
             with open(file, "rb") as f:
                 content = f.read()
 
